<?php
/**
 * Class for the report page.
 *
 * @package Freesoul Deactivate Plugins
 */

defined( 'ABSPATH' ) || exit; // Exit if accessed directly.

class Eos_Fdp_Report_Page extends Eos_Fdp_Plugins_Manager_Page {

  public $tool;
  public $speed_tools = array();
  public $opts;


  public function before_section( $page_slug ){
    $this->tool = isset( $_GET['tool'] ) ? sanitize_key( $_GET['tool'] ) : 'gtmetrix';
    $this->speed_tools = array( 'gtmetrix' => __( 'GTMetrix','eos-dp-pro'),'gpsi' => __( 'Google Page Speed Insights','eos-dp-pro' ) );
    $this->opts = eos_dp_pro_get_option( 'fdp_pro_admin' );

    if( !isset( $_REQUEST['export_file'] ) ){
    ?>
    <style>#wpcontent #eos-dp-report-section{min-width:0 !important}</style>
    <nav id="fdp-st-nav" class="no-print">
      <a class="eos-dp-no-decoration<?php echo 'gtmetrix' === $this->tool ? ' fdp-current' : ''; ?>" href="<?php echo admin_url( 'admin.php?page=eos_dp_report&tool=gtmetrix'); ?>"><span class="eos-dp-gtmetrix-icon"></span><?php echo esc_html( $this->speed_tools['gtmetrix'] ); ?></a> |
      <a class="eos-dp-no-decoration<?php echo 'gpsi' === $this->tool ? ' fdp-current' : ''; ?>" href="<?php echo admin_url( 'admin.php?page=eos_dp_report&tool=gpsi'); ?>"><span class="eos-dp-gpsi-icon"></span><?php echo esc_html( $this->speed_tools['gpsi'] ); ?></a>
    </nav>
    <?php
    }
  }
  public function get_nonces_map(){
    return array( 'eos_dp_report' => 'eos_dp_pro_report' );
  }
  public function tableBody( $page_slug ){
    $key_tool = 'last_'.$this->tool;
    if( !isset( $this->opts[$key_tool] ) ){
  	?>
  	<section id="eos-dp-report-section" class="eos-dp-section" style="min-width:0 !important">
  		<div><h2><?php esc_html_e( 'No tests found.','eos-dp-pro' ); ?></h2></div>
  	</section>
  	<?php
  	}
  	else{
  		global $wp_filesystem;
  		$upload_dir = wp_upload_dir();
  		$json = false;
  		$arr = array();
  		if( file_exists( $upload_dir['basedir'].'/FDP/reports/'.$this->tool.'/fdp-report-'.sanitize_title( $this->opts[$key_tool] ).'.json' ) ){
  			$access_type = get_filesystem_method();
  			if( $access_type === 'direct' ){
  				$creds = request_filesystem_credentials( admin_url(), '', false, false, array() );
  				if ( ! WP_Filesystem( $creds ) ) {
  					return false;
  				}
  				if( isset( $this->opts[$key_tool] ) ){
  					$json = $wp_filesystem->get_contents( $upload_dir['basedir'].'/FDP/reports/'.$this->tool.'/fdp-report-'.sanitize_title( $this->opts[$key_tool] ).'.json' );
  				}
  			}
  		}
  		if( $json && '' !== $json ){
  			$arr = json_decode( sanitize_text_field( $json ),true );
  		}
  		if( isset( $_GET['order_by'] ) && 'time' === $_GET['order_by'] ){
  			if( isset( $_GET['order'] ) ){
  				if( 'desc' === $_GET['order'] ){
  					krsort( $arr );
  				}
  				else{
  					ksort( $arr );
  				}
  			}
  		}
      if( isset( $_REQUEST['export_file'] ) ){ ?>
			<h2><?php esc_html_e( 'Speed test results.','eos-dp-pro' ); ?></h2>
			<h3><?php printf( esc_html__( 'Website URL: %s','eos-dp-pro' ),esc_url( home_url() ) ); ?></h3>
			<h3><?php printf( esc_html__( 'Testing tool: %s','eos-dp-pro' ),$this->speed_tools[$this->tool] ); ?></h3>
			<?php } ?>
			<?php if( !isset( $_GET['export_file'] ) ){ ?>
			<div style="margin:32px 0 32px 0">
				<?php if( isset( $this->opts[$key_tool] ) ){ ?>
				<a href="<?php echo  add_query_arg( 'tool',$this->tool,admin_url( 'admin.php?page=eos_dp_report&export_file=true' ) ); ?>" download="<?php echo sanitize_title( get_bloginfo( 'name' ) ).'-speed-tests.html'; ?>" id="eos-dp-download-test-table" class="button"><span class="dashicons dashicons-download" style="padding:4px"></span><?php esc_html_e( 'Download Table as HTML','eos-dp-pro' ); ?></a>
				<a id="eos-dp-print-report" href="#" class="button" title="<?php esc_attr_e( 'Print','eos-dp-pro' ); ?>" onclick="window.print();"><span class="dashicons dashicons-printer" style="padding:4px"></span><?php esc_html_e( 'Print','eos-dp-pro' ); ?></a>
				<?php } ?>
			</div>
			<?php }
			echo $this->report_table( $arr,true,admin_url( 'admin.php?page=eos_dp_report' ),$this->tool );
			if( !isset( $_GET['export_file'] ) ){
				eos_dp_save_button();
			}
			?>
			<script>var fdp_st_tool = "<?php echo esc_js( $this->tool ); ?>";</script>
  		<?php
  	}
  }
  public function tableHead( $page_slug ){

  }
  public function footer( $button_class ){

  }
  public function report_table( $arr,$sortable = false,$page_url = false,$tool = 'gtmetrix' ){
  	$testing_locations = eos_dp_pro_gtmetrix_locations();
  	$active_plugins = array_map( 'eos_dp_name_by_path',eos_dp_active_plugins() );
  	$active_plugins = array_combine( range( 1,count( $active_plugins) ),array_values( $active_plugins) );
  	$many_plugins = count( $active_plugins ) > 8;
  	$devices2icons = array( 'mobile' => 'smartphone','desktop' => 'desktop' );
  	$score_class = '';
  	$score_key = 'pagespeed_score';
  	if( 'gtmetrix' === $tool ){
  		$labels = eos_dp_pro_gtmetrix_labels();
  	}
  	elseif( 'gpsi' === $tool ){
  		$labels = eos_dp_pro_gpsi_labels();
  		$score_key = 'score';
  	}
  	if( $many_plugins ){
  		?>
  		<div class="eos-dp-legend" style="margin-bottom:16px">
  			<?php
  			$legend = '* ';
  			foreach( $active_plugins as $n => $plugin_name ){
  				$legend .= $n.': '.esc_attr( $plugin_name ).', ';
  			}
  			echo rtrim( $legend,', ' );
  			?>
  		</div>
  		<?php
  	}
  	?>
  	<table class="wp-list-table widefat fixed fdp-table-sticky-headers" cellspacing="0">
  		<thead>
  			<tr>
  				<th>
  					<?php esc_html_e( 'Time','eos-dp-pro' ); ?>
  					<?php
  					if( $sortable && $page_url ){
  						$url = add_query_arg( 'order','time',$page_url );
  						$this->order_arrows( $url,'time' );
  					}
  					?>
  				</th>
  			<?php
  			foreach( $labels as $key => $label ){ ?>
  				<th><?php echo esc_html( $label ); ?></th>
  			<?php } ?>
  			<?php
  			if( $sortable ){
  			?>
  			<?php
  			}
  			?>
  			</tr>
  		</thead>
  		<tbody class="<?php echo $sortable ? 'eos-dp-sortable-item-wrp' : ''; ?>">
  		<?php
  		$n = 0;
  		foreach( $arr as $time_key => $data ){
  			$date = date_i18n( 'F j,Y',$key );
  			$time = date_i18n( 'g:i a',$key );
  			if( isset( $data[$score_key] ) ){
  				$score = absint( $data[$score_key] );
  				if( $score < 50 ){
  					$score_class = ' fdp-st-low';
  				}
  				elseif( $score >= 50 && $score < 90 ){
  					$score_class = ' fdp-st-medium';
  				}
  				elseif( $score >= 90 ){
  					$score_class = ' fdp-st-high';
  				}
  			}
  		?>
  		<tr class="fdp-st-row<?php echo $score_class;echo $n % 2 === 0 ? ' alternate' : '';echo $sortable ? ' eos-dp-sortable-item' : ''; ?>" data-time="<?php echo esc_attr( $time_key ); ?>">
  			<td>
  				<?php echo esc_html( sprintf( '%s %s',$date,$time ) ); ?>
  				<?php
  				if( $sortable ){
  				?>
  				<div class="eos-dp-report-row-actions">
  					<span title="<?php esc_attr_e( 'Delete','eos-dp-pro' ); ?>" class="hover eos-dp-delete-row dashicons dashicons-trash"></span>
  					<span title="<?php esc_attr_e( 'Move','eos-dp-pro' ); ?>" class="dashicons dashicons-move"></span>
  				</div>
  				<?php
  				}
  				?>
  			</td>
  			<?php
  			foreach( $labels as $key => $label ){
  				if( 'page_url' === $key ){
  					$page_url = $data['page_url'];
  					$page_url = add_query_arg(
  						array(
  							'test_id' => sanitize_key( $time_key ),
  							'eos_dp_pro_id' => md5( EOS_DP_PRO_TESTING_UNIQUE_ID ),
  							'eos_dp_preview' => 'true',
  							'tool' => $tool
  						),$page_url
  					);
  					if( isset( $data['theme'] ) && 'false' !== $data['theme'] && '' !== $data['theme'] ){
  						$page_url = add_query_arg( array( 'theme' => esc_attr( $data['theme'] ) ),$page_url );
  					}
  					$value = '<a class="eos-dp-report-page-name" title="'.esc_attr( $data['post_title'] ).'" href="'.esc_url( $page_url ).'" target="_blank" rel="noopener">'.esc_html( $data['post_title'] ).'</a>';
  				}
  				elseif( 'disabled_plugins' === $key ){
  					if( isset( $data[$key] ) ){
  						if( $many_plugins ){
  							$disabled_plugins = explode( ',',$data[$key] );
  							foreach( $disabled_plugins as &$disabled_plugin ){
  								$disabled_plugin = array_search( $disabled_plugin,$active_plugins );
  							}
  							$value = '<span title="'.esc_attr( $data[$key] ).'">'.esc_html( implode( ',',$disabled_plugins ) ).'</span>';
  						}
  						else{
  							$value = esc_html( $data[$key] );
  						}
  					}
  					else{
  						$value = '';
  					}
  				}
  				elseif( 'device' === $key && isset( $devices2icons[$data[$key]] ) ){
  					$value = '<span class="dashicons dashicons-'.esc_attr( $devices2icons[$data[$key]] ).'"></span>';
  				}
  				elseif( 'location' === $key ){
  					$value = esc_html( $testing_locations[absint( $data[$key] )] );
  				}
  				elseif( 'report_url' === $key ){
  					$value = '<a title="'.esc_attr__( 'Link to test report','eos-dp-pro' ).'" href="'.esc_url( $data[$key] ).'" target="_blank" rel="noopener"><span class="dashicons dashicons-admin-links"></span></a>';
  				}
  				else{
  					$value = isset( $data[$key] ) ? esc_html( $data[$key] ) : '';
  				}
  			?>
  			<td class="fdp-<?php echo esc_attr( $key ); ?>"><?php echo $value; ?></td>
  			<?php
  				++$n;
  			}
  			?>
  		</tr>
  		<?php
  	}
  	?>
  		</tbody>
  	</table>
  	<?php
  }

  public function order_arrows( $url,$order_by ){
  	$order = isset( $_GET['order'] ) && 'asc' === $_GET['order'] ? 'asc' : 'desc';
  	$url = add_query_arg( 'order_by',$order_by,$url );
  ?>
  	<a title="<?php esc_attr_e( 'Descending order','eos-dp-pro' ); ?>" href="<?php echo add_query_arg( 'order','desc',$url ); ?>" class="eos-dp-order-rows hover dashicons dashicons-arrow-down<?php echo 'asc' !== $order ? ' eos-hidden' : ''; ?>"></a>
  	<a title="<?php esc_attr_e( 'Ascending order','eos-dp-pro' ); ?>" href="<?php echo add_query_arg( 'order','asc',$url ); ?>" class="eos-dp-order-rows hover dashicons dashicons-arrow-up<?php echo 'desc' !== $order ? ' eos-hidden' : ''; ?>"></a>
  <?php
  }
}
