<?php
/**
 * Class for the Who Is page.
 *
 * @package Freesoul Deactivate Plugins
 */

defined( 'ABSPATH' ) || exit; // Exit if accessed directly.

class FDP_Whois_Page extends Eos_Fdp_Plugins_Manager_Page {

  public $section_id;
  public $button_class;
  private $url;

  public function before_section( $page_slug ){
    $this->button_class = 'fdp-who-is';
    $this->section_id = 'eos-dp-who-is-section';
    ?>
    <style id="fdp-who-is-css"></style>
    <h2><?php _e( 'Source Checker.','eos-dp-pro' ); ?></h2>
		<div class="eos-dp-explanation" style="margin-bottom:48px">
			<p><?php _e( 'Ask FDP which plugin is printing that text or code that you see when you inspect the document.','eos-dp' ); ?></p>
		</div>
    <?php
  }
  public function section( $page_slug ){
    $active_plugins = array_merge( array( 'db','all','fdp_naked_theme' ),$this->active_plugins );
    wp_enqueue_script( 'fdp-whois',EOS_DP_PRO_PLUGIN_URL.'/admin/js/eos-dp-pro-whois.js',array(),EOS_DP_PRO_VERSION );
    wp_localize_script( 'fdp-whois','fdp_plugins_js',array(
      'ajaxurl' => admin_url( 'admin-ajax.php' ),
      'fdp_plugins_n' =>count( $active_plugins ),
      'fdp_plugins' => wp_json_encode( $active_plugins ),
      'not_found' => __( 'No plugin is the source. Maybe it is added by the core.','eos-dp-pro' ),
      'not_found_all' => __( 'Not found even with all plugins active. Maybe it is added by the core.','eos-dp-pro' ),
      'found' => __( 'It looks it was added by:','eos-dp-pro' ),
      'in_content' => __( 'Directly in the content.','eos-dp-pro' ),
    ) );
    wp_nonce_field( $page_slug.'_nonce',$page_slug.'_nonce' );
    ?>
    <section id="<?php echo esc_attr( $this->section_id ); ?>" class="<?php echo esc_attr( apply_filters( 'fdp_section_class_name','eos-dp-section' ) );echo ' '.esc_attr( $this->section_common_class ); ?>" data-page_slug="<?php echo esc_attr( $page_slug ); ?>">
      <?php do_action( 'eos_dp_before_wrapper' ); ?>
      <div id="eos-dp-wrp" style="max-width:900px">
  			<label for="fdp-whois-url"><?php _e( 'Page URL','eos-dp-pro' ); ?></label>
  			<p><input id="fdp-whois-url" type="text" class="regular-text" placeholder="<?php echo get_home_url(); ?>" value="<?php echo get_home_url(); ?>"/></p>
        <label for="fdp-whois-search-area"><?php _e( 'Search ()HTML, word, asset URL...)','eos-dp-pro' ); ?></label>
        <p><?php _e( 'Go to the page on frontend => Right-click => Inspect => Copy and then paste it into the text area.','eos-dp-pro' ); ?></p>
        <div style="background:#fff;padding:10px"><p><?php esc_html_e( 'Example:','eos-dp-pro' ); ?></p><?php echo esc_html( '<link rel="stylesheet" id="' ); ?><span style="background-color:yellow">ct-admin-frontend-styles-css</span><?php echo esc_html( '" href="'.get_home_url().'/wp-content/themes/blocksy/static/bundle/admin-frontend.min.css?ver=1.8.55" media="all">' ); ?></div>
        <p><textarea id="fdp-whois-search-area" class="widefat" style="width:100%" rows="10" placeholder="<?php echo esc_attr( sprintf( __( 'E.g.: %s','eos-dp-pro' ),'ct-admin-frontend-styles-css' ) ); ?>"></textarea></p>
        <div id="fdp-progress-bar" style="height:16px;width:0;background:rgba(37,48,66,0.5)"></div>
        <p style="margin-top:32px">
          <span id="fdp-whois-submit" class="button">
            <span class="dashicons dashicons-admin-plugins" style="margin:4px 0"></span>
            <?php esc_html_e( 'Search','eos-dp-pro' ); ?>
          </span>
        </p>
        <p><?php esc_html_e( 'Pressing the button, FDP will try to understand which plugin adds what you write in the textarea above.','eos-dp-pro' ); ?></p>

        <div id="fdp-success" class="eos-dp-notice notice notice-success eos-dp-opts-msg_success eos-hidden" style="padding:10px"><?php _e( 'Found plugin:','eos-dp-pro' ); ?></div>
    		<div id="fdp-fail" class="eos-dp-notice notice notice-error eos-hidden" style="padding:10px" data-default_msg="<?php esc_attr_e( 'Something went wrong!','eos-dp-pro' ); ?>"></div>

      </div>
    </section>
    <?php
  }
  public function tableBody( $page_slug ){
    return;
  }

  public function get_nonces_map(){
    return array(
      'eos_dp_whois' => 'eos_dp_whois'
    );
  }

  public function footer( $button_class ){
    require_once EOS_DP_PLUGIN_DIR.'/admin/templates/partials/eos-dp-footer.php';
  }
}
