<?php
defined( 'ABSPATH' ) || exit; // Exit if accessed directly

//Show assets globally removed in the website
function eos_dp_pro_assets_callback(){
  if( !current_user_can( 'activate_plugins' ) ){
  ?>
    <h2><?php esc_html_e( 'Sorry, you have not the right for this page','eos-dp' ); ?></h2>
    <?php
    return;
  }
  if( !isset( $_GET['fdp_in_iframe'] ) ){
    eos_dp_alert_plain_permalink();
    eos_dp_navigation();
  }
  else{
    ?><style id="fdp-global-view-iframe-css">#adminmenumain{display:none}#wpcontent{margin-left:initial}</style><?php
  }
  $opts = eos_dp_pro_get_option( 'eos_dp_pro_main' );
  $asset_type = isset( $_GET['asset_type'] ) ? esc_attr( $_GET['asset_type'] ) : 'styles';
	$excludes = array(
		'moved_header',
		'moved_footer',
		'reduced_styles',
    'custom_urls'
	);
	global $wpdb;
	$meta_key = '_fdp_'.$asset_type.'_active';
	$db_results = $wpdb->get_results( $wpdb->prepare( "SELECT post_id,meta_value FROM $wpdb->postmeta WHERE meta_key = %s", $meta_key) , ARRAY_N  );
	$single_active = array();
	foreach( $db_results as $db_result ){
		$handles = json_decode( stripslashes( $db_result[1] ),true );
		foreach( $handles as $handle ){
			$single_active[sanitize_key( $handle )][] = absint( $db_result[0] );
		}
	}
	?>
	<header style="margin-top:32px;margin-bottom:32px">
		<h2><?php echo 'scripts' === $asset_type ? esc_html__( 'Removed scripts','eos-dp-pro' ) : esc_html__( 'Removed styles','eos-dp-pro' ); ?></h2>
		<a class="button<?php echo 'styles' === $asset_type ? ' eos-active' : ''; ?>" href="<?php echo admin_url( 'admin.php?page=eos_dp_pro_assets&asset_type=styles' ); ?>"><?php esc_html_e( 'Styles','eos-dp-pro' ); ?></a>
		<a class="button<?php echo 'scripts' === $asset_type ? ' eos-active' : ''; ?>" href="<?php echo admin_url( 'admin.php?page=eos_dp_pro_assets&asset_type=scripts' ); ?>"><?php esc_html_e( 'Scripts','eos-dp-pro' ); ?></a>
	</header>
	<?php
  if( isset( $opts[$asset_type] ) && !empty( $opts[$asset_type] ) ){
    $opts_asset_type = json_decode( sanitize_text_field( stripslashes( $opts[$asset_type]) ),true );
    $custom_urls = isset( $opts_asset_type['custom_urls'] ) ? json_decode( sanitize_text_field( stripslashes( str_replace( '\n','',$opts_asset_type['custom_urls'] ) ) ),true ) : false;
    ?>
    <table id="fdp-site-global-view-assets" class="wp-list-table widefat striped table-view-list" data-asset-type="<?php echo $asset_type; ?>">
      <thead>
        <tr>
          <th><?php esc_html_e( 'Asset ID','eos-dp-pro' ); ?></th>
          <th><?php esc_html_e( 'Removed in','eos-dp-pro' ); ?></th>
          <th><?php esc_html_e( 'Active in','eos-dp-pro' ); ?></th>
          <th><?php esc_html_e( 'Removed by custom URLs','eos-dp-pro' ); ?></th>
        </tr>
      </thead>
      <tbody>
        <?php
        foreach( $opts_asset_type as $asset_id => $arr ){
          if( in_array( $asset_id,$excludes ) ) continue;
        ?>
        <tr class="fdp-asset-tr" data-asset-id="<?php echo esc_attr( $asset_id ); ?>">
          <td>
						<span data-type="asset_id"><?php echo esc_html( $asset_id ); ?></span>
					</td>
          <td>
            <table>
							<tbody>
              <?php
							foreach( $arr as $k => $values ){
								if( empty( $values ) || !is_string( $values ) ) continue;
								$vals = explode( ',',rtrim( ltrim( $values,',' ),',' ) );
                if( empty( $vals ) ) continue;
								?>
	              <tr>
	                <td style="padding-top:0">
										<?php
                    if( 'single' === $k ){
                      foreach( $vals as $id ){
                        $post = get_post( $id );
                        if( $post ){
                        ?>
                        <span title="<?php echo esc_attr( sprintf( __( 'Post ID: %s. Post type: %s','eos-dp-pro' ),$id,$post->post_type ) ); ?>" class="fdp-asset-s" data-type="removed_in" data-where_type="<?php echo esc_attr( $id ); ?>" data-post_type="<?php echo esc_attr( $post->post_type ); ?>">
                          <?php echo esc_html( $post->post_title ); ?>
                        </span>
                      <?php
                        }
                      } }
										foreach( $vals as $val ){
											if( $val ){
												if( 'post_types' === $k ){
													$ptObj = get_post_type_object( $val );
													$ids = get_posts( array( 'post_type' => $val,'posts_per_page' => 1,'field' => 'ID' ) );
													$labs = $ptObj->labels;
													$pt_name = isset( $labs->name ) ? $labs->name : $val;
													?>
													<span title="<?php echo esc_html( $pt_name ); ?>" class="fdp-asset-pt" data-type="removed_in" data-where_type="<?php echo esc_attr( $k ); ?>" data-post_type="<?php echo esc_attr( $val ); ?>">
														<?php echo esc_html( $pt_name ); ?>
													</span>
												<?php }
												elseif( 'archives' === $k ){
													$url = add_query_arg( array( 'fdp-assets' => $asset_type,'t' => time() ),get_post_type_archive_link( $val ) );
													$ptObj = get_post_type_object( $val );
													$labs = $ptObj->labels;
													$pt_name = isset( $labs->name ) ? $labs->name : $val;
													?>
													<span title="<?php echo esc_html( $pt_name ); ?>" class="fdp-asset-s" data-archive="<?php echo esc_attr( $val ); ?>">
														<a href="<?php echo $url; ?>" target="_blank"><?php echo esc_html( $pt_name ); ?></a>
													</span>
												<?php
												}
											}
										} ?>
									</td>
								</tr>
							<?php
              }
              ?>
							</tbody>
            </table>
          </td>
					<td>
						<?php
            if( isset( $single_active[$asset_id] ) ){
  						foreach( $single_active[$asset_id] as $id ){
  							?>
  							<span class="fdp-asset-s-id" data-id="<?php echo esc_attr( $id ); ?>">
  								<a href="<?php echo add_query_arg( array( 'fdp-assets' => $asset_type,'t' => time() ),get_permalink( absint( $id ) ) ); ?>" target="_blank"><?php echo esc_html( get_the_title( $id ) ); ?></a>
  							</span>
  						<?php
  						}
            }
						?>
					</td>
          <td>
          <?php if( $custom_urls && is_array( $custom_urls ) && !empty( $custom_urls ) && in_array( $asset_id,array_keys( $custom_urls ) ) ){
            $urls_string = implode( '&nbsp;',explode( ',',$custom_urls[$asset_id] ) );
            echo str_replace( '&nbsp;','<br/>',esc_html( $urls_string ) );
          }
          ?>
          </td>
        </tr>
        <?php
        }
        foreach( $custom_urls as $url_handle => $urls ){
          if( $opts_asset_type && is_array( $opts_asset_type ) && !empty( $opts_asset_type ) && in_array( $url_handle,array_keys( $opts_asset_type ) ) ) continue;
          $urls_string = implode( '&nbsp;',explode( ',',$urls ) );
          ?>
          <tr class="fdp-asset-tr">
            <td><span><?php echo esc_html( $url_handle ); ?></span></td>
            <td></td>
            <td></td>
            <td><?php echo str_replace( '&nbsp;','<br/>',esc_html( $urls_string ) ); ?></td>
          </tr>
          <?php
        }
      ?>
      </tbody>
    </table>
    <?php
  }
	else{
		?>
		<p><?php esc_html_e( 'No asset removed','eos-dp-pro' ); ?></p>
		<?php
	}
}
