<?php
defined( 'ABSPATH' ) || exit; // Exit if accessed directly
function eos_dp_pro_general_bloat_callback(){
  if( !current_user_can( 'activate_plugins' ) && !current_user_can( 'fdp_plugins_viewer' ) ){
  ?>
    <h2><?php esc_html_e( 'Sorry, you have not the right for this page','eos-dp' ); ?></h2>
    <?php
    return;
  }
  $opts = eos_dp_pro_get_option( 'eos_dp_pro_main' );
  $opts = isset( $opts['general_bloat'] ) ? $opts['general_bloat'] : array();
  wp_nonce_field( 'fdp_general_bloat_nonce','fdp_general_bloat_nonce' );
  $unneeded_actions = eos_dp_pro_unneeded_actions();
  $unneeded_actions['sworg_dns_prefetch'] = array( false,false,__( 'S.W.ORG DNS prefetch','eos-dp-pro' ),null,null,__( "You need it only if on the frontend you load files from the WP repository. If unchecked it will be removed everywhere","eos-dp-pro" ) );
  $unneeded_actions['xmlrpc'] = array( false,false,__( 'XML-RPC','eos-dp-pro' ),null,null,__( 'You need it if you want to standardize communication between different systems, meaning that applications outside WordPress. If unchecked XML-RPC will be completely disabled','eos-dp-pro' ) );
  $unneeded_actions['rest_api'] = array( false,false,__( 'Rest API non-logged users','eos-dp-pro'),null,null,__( 'If unchecked the Rest API will be disabled, but onyl for non-logged users','eos-dp-pro' ) );
  $unneeded_actions['heartbeat'] = array( false,false,__( 'Heartbeat','eos-dp-pro'),null,null,__( 'If unchecked the Heartbeat will be completely disabled everywhere. Increasing the number you can reduce the tick of the heartbeat. The default value is 60. The minimum is 15 and the maximum is 120. If you try a number higher than 120 WordPress will use 120, and for numbers lower than 15 it will use 15.','eos-dp-pro' ) );
  $unneeded_actions['dirsize_cache'] = array( false,false,__( 'Tranient dirsize_cache','eos-dp-pro'),null,null,__( 'If unchecked the dirsize_cache will be completely disabled. This transient may drammatically increase the size of the autoloaded options, and you need it only to calculate faster the disk space of the upload folder, but only for debugging. Better you disable it.','eos-dp-pro' ) );
  $unneeded_actions['revisions'] = array( false,false,__( 'Revisions','eos-dp-pro'),null,null,__( 'If unchecked the revisions will be completely disabled for every post type. You can also specify a maximum number of revisions. IMPORTANT: it will limit the number of revisions but will not delete the ones that have already been created in excess.','eos-dp-pro' ) );
  $unneeded_actions['comment_spam'] = array( false,false,__( 'Comment Spam','eos-dp-pro'),null,null,__( "Unchecking this FDP will try to stop all the comment spam of your blog posts. You don't need that trash in your database. You need to manually delete all the spams that were saved before using this feature.","eos-dp-pro" ) );
  $unneeded_actions['gutenberg'] = array( false,false,__( 'Gutenberg','eos-dp-pro'),null,null,__( 'If unchecked Gutenberg will be completely disabled both on backend and frontend, and the related script and stylesheet will not be loaded','eos-dp-pro' ) );
  $unneeded_actions['global_style'] = array( false,false,__( 'Global style','eos-dp-pro'),null,null,__( 'Since version 5.9 WordPress adds a huge inline style that many website do not need at all. Even themes that do not use theme.json will load global style. If you don\'t need it, uncheck it to remove this core bloat.','eos-dp-pro' ) );
  $unneeded_actions['cron'] = array( false,false,__( 'WP Cron','eos-dp-pro'),null,null,__( "If unchecked the internal Cron of WordPress will be completely disabled. If you have an external cron you can disable it, but you should know what you are doing. If you don't know what is it, keep it active.","eos-dp-pro" ) );
  $unneeded_actions['autosave'] = array( false,false,__( 'Autosave','eos-dp-pro'),null,null,__( 'If unchecked the autosave of posts will be completely disabled','eos-dp-pro' ) );
  $unneeded_actions['cookies_assets'] = array( false,false,__( 'Cookies from static resources','eos-dp-pro'),'miscellaneus',__( 'Miscellaneus','eos-dp-pro' ),__( 'If unchecked FDP will prevent modern browser from sending cookies when they request static resources','eos-dp-pro' ) );
  $unneeded_actions['gfonts'] = array( false,false,__( 'Google Fonts','eos-dp-pro'),'general-assets',__( 'General assets','eos-dp-pro' ),__( 'If unchecked FDP will try to remove all the Google fonts loaded by the theme and other plugins and replace them with system fonts','eos-dp-pro' ) );
  $unneeded_actions['favicon'] = array( false,false,__( 'Deactivate all plugins and the theme if favicon.ico not found','eos-dp-pro'),null,null,sprintf( __( "If your theme calls %s/favicon.ico, and WordPress doesn't find it, it will load all the plugins and the theme before redirecting %s/favicon.ico to the final image URL. If you uncheck this checkbox, FDP will unload all the plugins and the theme before this redirection. In any case, it would be better that you put favicon.ico in the main directory to avoid this expensivve redirection.",'eos-dp-pro' ),get_home_url(),get_home_url().'/favicon.ico' ) );
  $unneeded_actions['jquery'] = array( false,false,__( 'jQuery in header','eos-dp-pro' ),null,null,__( 'If unchecked jQuery will be moved to the footer where possible','eos-dp-pro' ) );
  $unneeded_actions['jmigrate'] = array( false,false,__( 'jQuery Migrate','eos-dp-pro' ),null,null,__( 'You need jQuery migrate only if a plugin or the theme has some old jQuery. If unchecked it will not be loaded on the frontend','eos-dp-pro' ) );
  $unneeded_actions['emoji'] = array( false,false,__( 'Emoji','eos-dp-pro'),null,null,__( "You need Emoji only if you want to use the WP core script and stylesheet to display the emoticons, usualy you don't need it","eos-dp-pro" ) );
  if( function_exists( 'fdp_is_plugin_globally_active' ) && fdp_is_plugin_globally_active( 'woocommerce/woocommerce.php' ) ){
    $unneeded_actions['cart_fragments'] = array( false,false,__( 'Cart fragments','eos-dp-pro'),'wooocommerce',__( 'WooCommerce','eos-dp-pro' ),__( "It's a script loaded by WooCommerce to update the cart content without refreshing the page. If unchecked here, it will not be loaded anywhere. If you need it on some pages, better you manage it with the FDP JS manager","eos-dp-pro" ) );
    $unneeded_actions['woo_emails'] = array( false,false,__( 'Transactional emails while processing the order.','eos-dp-pro'),null,null,__( "WooCommerce sends transactional emails immediately after the user places an order. Unchecking this checkbox the transactional emails will be sent after processing the order, not during the process. The user will have the impression of a faster checkout.","eos-dp-pro" ) );
    $unneeded_actions['woo_blocks_style'] = array( false,false,__( 'WooCommerce Blocks Style.','eos-dp-pro'),null,null,__( "If you don't use WooCommerce Blocks, you don't need this bloat.","eos-dp-pro" ) );
  }
  $pages = get_posts( array( 'post_type' => 'page','posts_per_page' => 1 ) );
  $page_id = $pages && isset( $pages[0] ) && is_object( $pages[0] ) ? $pages[0]->ID : false;
  $html_markup = array(
 	 'rsd_link' => '<link rel="EditURI" type="application/rsd+xml" title="RSD" href="' . esc_url( site_url( 'xmlrpc.php?rsd', 'rpc' ) ) . '" />',
 	 'wp_shortlink_wp_head' => $page_id ? '<link rel="shortlink" href="'.wp_get_shortlink( $page_id ).'">' : '<link rel="shortlink" href="'.get_home_url().'">',
 	 'wp_generator' => get_the_generator( 'xhtml' ),
 	 'wlwmanifest_link' => '<link rel="wlwmanifest" type="application/wlwmanifest+xml" href="' . includes_url( 'wlwmanifest.xml' ) . '" /> ',
 	 'rest_output_link_wp_head' => sprintf( '<link rel="https://api.w.org/" href="%s" />', esc_url( get_rest_url() ) ),
 	 'rest_output_link_header' => sprintf( 'Link: <%s>; rel="https://api.w.org/"', esc_url_raw( get_rest_url() ) ),
   'sworg_dns_prefetch' => '<link rel="dns-prefetch" href="//s.w.org">',
   'xmlrpc' => '<link rel="EditURI" type="application/rsd+xml" title="RSD" href="' . esc_url( site_url( 'xmlrpc.php?rsd', 'rpc' ) ) . '" />'
 	);
	eos_dp_navigation();
	?>
  <section id="eos-dp-general-bloat-section" class="eos-dp-section fdp-pro-section">
    <h2 class="left"><?php _e( "Uncheck anything you don't need.","eos-dp-pro" ); ?></h2>
    <table id="eos-dp-setts">
      <tr id="fdp-gen-wp"><th colspan="2"><h2><?php esc_html_e( 'Core','eos-dp-pro' ); ?></h2></tr>
      <?php foreach( $unneeded_actions as $k => $arr ){
      $checked = !isset( $opts[$k] ) || 'false' !== $opts[$k] ? ' checked' : '';
      $html = isset( $html_markup[$k] ) ? $html_markup[$k] : false;
      if( isset( $arr[3] ) && isset( $arr[4] ) ){
        ?>
        <tr id="fdp-gen-<?php echo esc_attr( $arr[3] ); ?>"><th colspan="2"><h2><?php echo esc_html( $arr[4] ); ?></h2></tr>
        <?php
      }
      ?>
      <tr>
        <th>
          <?php
          echo esc_html( $arr[2] );

          if( 'heartbeat' === $k ){
            $heartbeat_value = isset( $opts['heartbeat_value'] ) ? $opts['heartbeat_value'] : 60;
          ?>
          <input id="heartbeat_value" class="fdp-opt" name="heartbeat_value" type="number" min="15" max="120" step="10" value="<?php echo absint( $heartbeat_value ); ?>" />
          <?php
          }
          if( 'revisions' === $k ){
            $revisions_value = isset( $opts['revisions_value'] ) ? $opts['revisions_value'] : 9999;
          ?>
          <input id="revisions_value" class="fdp-opt" name="revisions_value" type="number" min="0" max="200" step="1" value="<?php echo absint( $revisions_value ); ?>" />
          <?php
          }

          if( !isset( $arr[5] ) ){
          ?>
          <a class="eos-dp-no-decoration" href="<?php echo add_query_arg( 'q',urlencode( 'wordpress '.esc_attr( $arr[2] ) ),'https://google.com/search' ); ?>" target="_blank" rel="noopener"><span class="dashicons dashicons-editor-help"></span></a>
        <?php }else{ ?>
          <a class="eos-dp-no-decoration fdp-has-tooltip" href="#">
            <span class="dashicons dashicons-editor-help"></span>
            <div class="fdp-tooltip">
              <?php
              echo esc_html( $arr[5] );
              if( $html ){ ?>
              <p style="margin-top:16px"><?php esc_html_e( 'How it looks:','eos-dp-pro' ); ?></p>
              <p style="padding:10px;border:1px solid;background: rgba(255,255,255,0.7);color:#000"><?php echo esc_html( $html ); ?></p>
              <?php
              }
              ?>
            </div>
          </a>
        <?php } ?>
        </th>
        <td class="<?php echo ' checked' === $checked ? 'eos-dp-active' : ''; ?>">
          <div class="eos-dp-td-chk-wrp">
            <input id="fdp-<?php echo $k; ?>" class="fdp-gbloat-chk" type="checkbox" data-key="<?php echo $k; ?>" value="1"<?php echo $checked; ?> />
          </div>
        </td>
      </tr>
      <?php } ?>
    </table>
  </section>
  <?php
  if( current_user_can( 'activate_plugins' ) ){
    eos_dp_save_button();
  }
  ?>
  <div id="eos-dp-overlap"></div>
	<?php
}
