<?php
defined( 'ABSPATH' ) || exit; // Exit if accessed directly.
/*
* It shows the assets control panel.
*/
class FDP_PRO_Live_Preview {
	private $status;
	private $post_id = false;
	private $page_title = '';
	private $assets = array();
	private $assets_footer = array();
	private $assets_in_footer = array();
	private $options_assets = array();
	private $reduced_styles = array();

	public function __construct() {
		$this->set_active_status();
		$options = eos_dp_pro_get_option( 'eos_dp_pro_main' );
		$this->options_assets = isset( $options[$this->status] ) ? json_decode( sanitize_text_field( stripslashes( $options[$this->status] ) ),true ) : array();
		add_action( 'wp_enqueue_scripts', array( $this,'enqueue_script' ),PHP_INT_MAX );
		add_action( 'wp_head', array( $this,'init' ) );
		add_action( 'wp_footer', array( $this,'render_footer'),PHP_INT_MAX );
		add_filter( 'body_class', array( $this,'body_class' ) );
	}

	function init(){
		if( !isset( $_GET['fdp-live'] ) || !current_user_can( 'activate_plugins' ) ) {
			$this->status = 'off';
			return;
		}
		?>
		<style id="fdp-manage-links">
		body a{pointer-events:none !important}
		</style>
		<script>
		function eos_dp_pro_add_query_argument(a){
			if(a.href.length > 2 && a.href.indexOf(location.hostname) !== -1){
				if(a.href.indexOf('fdp-assets') < 0){
					var b = a.href.indexOf('?') > 0 ? '&' : '?',new_href = a.href + b + 'fdp-assets=<?php echo esc_js( esc_attr( $_GET['fdp-assets'] ) ); ?>';
					window.location.href = new_href.split('&t=')[0] + '&t=' + Date.now();
				}
			}
		}
		</script>
		<?php
	}
	//It sets the active status of the assets displaying.
	public function set_active_status() {
		if ( ! isset( $this->status ) ) {
			if( isset( $_REQUEST['fdp-live'] ) ) {
				$this->status = esc_attr( $_REQUEST['fdp-live'] );
			}
			else{
				$this->status = 'off';
			}
		}
	}

	public function udpate_links( $nav_menu,$args ){
		?>
		<script>
		setInterval(function(){
			eos_dp_pro_add_query_argument();
		},300);
		</script>
		<?php
	}
	public function body_class( $classes ){
		$classes[] = 'fdp-assets-page';
		return $classes;
	}

	//Enqueue Scripts
	public function enqueue_script(){
		global $post,$wp_styles;
		$this->post_id = is_object( $post ) && $post->ID  ? absint( $post->ID ) : 0;
		if( is_front_page() ) $this->post_id = get_site_option( 'page_on_front' );
		wp_enqueue_style( 'fdp-assets', EOS_DP_PRO_PLUGIN_URL. '/assets/css/fdp-show-assets.css','',EOS_DP_PRO_VERSION,'all' );
		wp_enqueue_style( 'fdp-dashicons',$wp_styles->registered['dashicons']->src );
		if( is_user_logged_in() ){
			wp_enqueue_script( 'fdp-assets-js', EOS_DP_PRO_PLUGIN_URL . '/assets/js/fdp-show-assets.js', array( 'jquery','jquery-ui-draggable'),EOS_DP_PRO_VERSION,true );
			wp_localize_script('fdp-assets-js', 'fdp_assets', array(
				'post_id' => absint( $this->post_id ),
				'post_type' => get_post_type( absint( $this->post_id ) ),
				'warning_style_needed' => __( 'Are you sure?. It looks this stylesheet has an impact on the style!'),
				'ajaxurl' => admin_url('admin-ajax.php'),
				'asset' => $this->status
			));
		}
	}

	function render_footer(){
		global $post;
		$this->assets_control_panel();
		if( is_user_logged_in() ){
		?>
			<script>
			var fdpa = document.getElementsByTagName('a');
			for(var n=0;n<fdpa.length;++n){
				fdpa[n].addEventListener('click',function(e){
					if('undefined' === typeof(this.className) ||  this.className.indexOf('fdp-page-view') < 0){
						e.preventDefault();
						eos_dp_pro_add_query_argument(this);
						return false;
					}
				});
			}
			</script>
			<?php } ?>
			<div id="fdp-assets-footer" class="fdp-assets-wrp">
				<?php
				$this->render_assets( $this->status,'footer' );
				?>
			</div>
		</div>
		<?php
	}

	function assets_control_panel(){
		if( isset( $_GET['fdp-in-iframe'] ) || !isset( $_GET['fdp_post_type'] ) ) return;
		$post_type = sanitize_key( $_GET['fdp_post_type'] );
		$this->page_title = absint( $this->post_id ) > 0 ? esc_html( get_the_title( $this->post_id ) ) : '';
		$post_types_matrix = eos_dp_get_option( 'eos_post_types_plugins' );
		$eos_dp_paths = '';
		$post_meta = get_post_meta( absint( $this->post_id ),'_eos_deactive_plugins_key',true );
		$post_types_matrix_pt = isset( $post_types_matrix[$post_type] ) ? $post_types_matrix[$post_type] : false;
		if( $post_types_matrix_pt && isset( $post_types_matrix_pt[0] ) && '0' == $post_types_matrix_pt[0] ){
			$eos_dp_paths = explode( ',',$post_meta );
		}
		else{
			if( $post_types_matrix && isset( $post_types_matrix[$post_type] ) ){
				if( isset( $post_types_matrix_pt[3] ) ){
					$ids = $post_types_matrix_pt[3];
					if( in_array( absint( $this->post_id ),$ids ) ){
						$eos_dp_paths = explode( ',',$post_meta );
						//'Plugins disabled according to Singles Settings
					}
					else{
						$eos_dp_paths = explode( ',',$post_types_matrix_pt[1] );
						//Plugins disabled according to the Post Types settings.
					}
				}
				else{
					$eos_dp_paths = explode( ',',$post_types_matrix_pt[1] );
					//Settings overridden by the Post Types Settings
				}
			}
		}

		require_once EOS_DP_PLUGIN_DIR.'/admin/eos-dp-helper.php';
		?>
		<div id="fdp-assets-main-wrp" style="left:-300px">
			<div id="fdp-assets-content">
				<h2><a class="fdp-page-view" href="<?php the_permalink(); ?>" target="_blank"><?php the_title(); ?></a></h2>
				<div class="fdp-save-wrp">
					<span class="fdp-assets-save fdp-button-active fdp-assets-button" data-action="eos_dp_pro_save_assets" data-nonce="fdp-save-assets" data-action-id="<?php echo uniqid(); ?>"><?php esc_html_e( 'Save','fdp-pro' ); ?></span>
					<div id="fdp-message-success" class="fdp-message fdp-success fdp-hidden"><?php esc_html_e( 'Options saved successfully.','eos-dp-pro' ); ?></div>
					<div id="fdp-message-error" class="fdp-message fdp-error fdp-hidden"><?php esc_html_e( 'Something went wrong.','eos-dp-pro' ); ?></div>
				</div>
				<div class="fdp-assets-control-panel">
					<?php
					if( 'off' === $this->status ){
						?>
						<div class="fdp-notice fdp-error" style="margin-top:32px">
							<?php esc_html_e( "You haven't the rights to manage these options!","eos-dp-pro" ); ?>
							<p style="margin-top:32px"><a class="fdp-button fdp-button-active" href="<?php echo wp_login_url( eos_dp_get_current_page_url() ); ?>" style="padding:6px 4px"><?php esc_html_e( 'Log in','eos-dp-pro' ); ?></a></p>
						</div>
					<?php }else{
					?>
					<div style="margin-bottom:16px">
						<span id="fdp-assets-filter-global-view" class="fdp-needs-ajax fdp-assets-button">
							<a href="#"><?php esc_html_e( 'Global View','fdp-pro' ); ?></a>
						</span>
						<span id="fdp-assets-filter-close" class="fdp-assets-button">
							<a href="#"><?php esc_html_e( 'Close','fdp-pro' ); ?></a>
						</span>
					</div>
					<div style="margin-bottom:32px;margin-top:16px">
						<h1 id="fdp-page-title"><?php echo esc_html( $this->page_title ); ?></h1>
						<span id="fdp-collapse-sidebar" class="fdp-move-sidebar dashicons dashicons dashicons-arrow-left" title="<?php esc_attr_e( 'Collapse','eos-dp-pro' ); ?>"></span>
						<span id="fdp-expand-sidebar" class="fdp-move-sidebar dashicons dashicons-arrow-right" title="<?php esc_attr_e( 'Expand','eos-dp-pro' ); ?>"></span>
					</div>
					<div>
						<p style="margin-bottom:16px">
							<span id="fdp-assets-filter-all" class="fdp-assets-filter fdp-button-active fdp-assets-button" data-filter="all"><?php esc_html_e( 'All','fdp-pro' ); ?></span>
						</p>
						<div style="margin-bottom:16px" id="fdp-plugins-wrp">
							<?php
							$active_plugins = eos_dp_active_plugins();
							foreach( $active_plugins as $plugin ){
								$plugin_slug = dirname( $plugin );
								$plugin_name = ucwords( str_replace( '-',' ',$plugin_slug ) );
								$is_active = $eos_dp_paths && is_array( $eos_dp_paths ) && in_array( $plugin,$eos_dp_paths );
								?>
								<div class="fdp-asset<?php echo $is_active ? '' : ' fdp-active'; ?>" data-path="<?php echo esc_attr( $plugin ); ?>">
									<?php echo esc_html( $plugin_name ); ?>
								</div>
								<?php
							}
							?>
						</div>
						<div id="fdp-assets-global-view-iframe-wrp" class="eos-hidden">
							<p class="right" style="margin-bottom:-10px;padding-top:10px"><span id="fdp-close-assets-iframe" class="hover dashicons dashicons-no-alt"></span></p>
							<iframe id="fdp-assets-global-view-iframe" src="?fdp-in-iframe=true" data-src="<?php echo add_query_arg( array( 'page' => 'eos_dp_pro_assets','asset_type' => $this->status,'fdp_in_iframe' => 'true' ),admin_url( 'admin.php' ) ); ?>"></iframe>
						</div>
					</div>
				<?php } ?>
				</div>
		<?php
		wp_nonce_field( 'fdp-save-assets','fdp-save-assets' );
	}

	function render_assets( $type,$position ){
		?>
		<script>
		var fdp_body = document.getElementsByTagName('body')[0];
		fdp_body.className = fdp_body.className.replace( ' fdp-assets-loaded','') + ' fdp-assets-loaded';
		document.getElementById('fdp-manage-links').innerHTML = '.fdp-assets-loaded a{pointer-events:initial}';
		</script>
		<?php
	}

	function get_assets( $type ){
		if( !isset( $GLOBALS['wp_'.$type] ) ) return array();
		$assets = array();
		$exclude = array(
			'admin-bar',
			'fdp-dashicons',
			'fdp-assets-js'
		);
		foreach( $GLOBALS['wp_'.$type]->queue as $asset ){
			if( !in_array( $asset,$exclude ) ){
			 $assets[$asset] =  $GLOBALS['wp_'.$type]->registered[$asset]->src;
		 	}
		}
		return $assets;
	}
}
$fdp_assets = new FDP_PRO_Live_Preview();
