<?php
defined( 'ABSPATH' ) || exit;
/*
* It shows the assets control panel.
*/
class FDP_PRO_Show_Assets {
	private $status;
	private $post_id = false;
	private $page_title = '';
	private $assets = array();
	private $assets_head = array();
	private $assets_footer = array();
	private $assets_in_footer = array();
	private $options_assets = array();
	private $reduced_styles = array();

	public function __construct() {
		$this->set_active_status();
		$options = eos_dp_pro_get_option( 'eos_dp_pro_main' );
		$this->options_assets = isset( $options[$this->status] ) && is_string( $options[$this->status] ) ? json_decode( sanitize_text_field( stripslashes( $options[$this->status] ) ),true ) : array();
		add_action( 'wp_enqueue_scripts', array( $this,'enqueue_script' ),PHP_INT_MAX );
		add_action( 'wp_head', array( $this,'init' ) );
		add_action( 'plugins_loaded', array( $this,'solve_conflicts' ) );
		add_action( 'wp_head', array( $this,'assets_head'),PHP_INT_MAX );
		add_action( 'wp_footer', array( $this,'render_footer'),PHP_INT_MAX );
		add_filter( 'body_class', array( $this,'body_class' ) );
	}

	function init(){
		if( !isset( $_GET['fdp-assets'] ) || !isset( $GLOBALS['wp_'.$this->status] ) || !current_user_can( 'activate_plugins' ) ) {
			$this->status = 'off';
			return;
		}
		?>
		<style id="fdp-manage-links">body a{pointer-events:none !important}</style>
		<script>
		function eos_dp_pro_add_query_argument(a){
			if(a.href.length > 2 && a.href.indexOf(location.hostname) !== -1){
				if(a.href.indexOf('fdp-assets') < 0){
					var b = a.href.indexOf('?') > 0 ? '&' : '?',new_href = a.href + b + 'fdp-assets=<?php echo esc_js( esc_attr( $_GET['fdp-assets'] ) ); ?>';
					window.location.href = new_href.split('&t=')[0] + '&t=' + Date.now();
				}
			}
		}
		</script>
		<?php
	}
	//It sets the active status of the assets displaying.
	public function set_active_status() {
		if ( ! isset( $this->status ) ) {
			if( isset( $_REQUEST['fdp-assets'] ) ) {
				$this->status = esc_attr( $_REQUEST['fdp-assets'] );
			}
			else{
				$this->status = 'off';
			}
		}
	}

	public function solve_conflicts(){
		add_filter( 'wp_hummingbird_modules',function( $modules ){
			// Solve conflict with Hummingbird.
			if( in_array( 'minify',$modules ) ){
				unset( $modules[array_search( 'minify',$modules )] );
			}
			return $modules;
		} );
	}

	public function udpate_links( $nav_menu,$args ){
		?>
		<script>
		setInterval(function(){
			eos_dp_pro_add_query_argument();
		},300);
		</script>
		<?php
	}
	public function body_class( $classes ){
		$classes[] = 'fdp-assets-page';
		return $classes;
	}

	//Enqueue Scripts
	public function enqueue_script(){
		global $wp_styles;
		if( is_archive() ){
			if( function_exists( 'is_shop' ) && is_shop() ){
				$this->post_id = eos_dp_pro_get_option( 'woocommerce_shop_page_id' );
			}
		}
		else{
			global $post;
			$this->post_id = is_object( $post ) && $post->ID  ? absint( $post->ID ) : 0;
		}
		if( is_front_page() ) $this->post_id = get_site_option( 'page_on_front' );
		wp_enqueue_style( 'fdp-assets', EOS_DP_PRO_PLUGIN_URL. '/assets/css/fdp-show-assets.css','',EOS_DP_PRO_VERSION,'all' );
		wp_enqueue_style( 'fdp-dashicons',$wp_styles->registered['dashicons']->src );
		if( is_user_logged_in() ){
			wp_enqueue_script( 'fdp-assets-js', EOS_DP_PRO_PLUGIN_URL . '/assets/js/fdp-show-assets.js', array( 'jquery','jquery-ui-draggable'),EOS_DP_PRO_VERSION,true );
			wp_localize_script('fdp-assets-js', 'fdp_assets', array(
				'post_id' => absint( $this->post_id ),
				'post_type' => get_post_type( absint( $this->post_id ) ),
				'warning_style_needed' => __( 'Are you sure?. It looks this stylesheet has an impact on the style!'),
				'ajaxurl' => admin_url('admin-ajax.php'),
				'asset' => $this->status
			));
		}
	}

	function assets_head(){
		global $wp_scripts;
		$this->assets_head = array_keys( $wp_scripts->registered );
		$this->assets_in_footer = isset( $wp_scripts->in_footer ) ? $wp_scripts->in_footer : array();
	}

	function render_footer(){
		global $post,$wp_scripts;
		$this->assets_footer = array_keys( $wp_scripts->registered );
		$in_footer = array();
		foreach( $this->assets_footer as $handle ){
			if( !in_array( $handle,$this->assets_head ) ){
				$in_footer[] = $handle;
			}
		}
		$this->assets_in_footer = array_unique( array_merge( $this->assets_in_footer,$in_footer ) );
		$this->assets = $this->get_assets( $this->status );
		$this->assets_control_panel();
		if( is_user_logged_in() ){
		?>
			<script>
			var fdpa = document.getElementsByTagName('a');
			for(var n=0;n<fdpa.length;++n){
				fdpa[n].addEventListener('click',function(e){
					if('undefined' === typeof(this.className) ||  this.className.indexOf('fdp-page-view') < 0){
						e.preventDefault();
						eos_dp_pro_add_query_argument(this);
						return false;
					}
				});
			}
			</script>
			<?php } ?>
			<div id="fdp-assets-footer" class="fdp-assets-wrp">
				<?php
				$this->render_assets( $this->status,'footer' );
				?>
			</div>
		</div>
		<?php
	}

	function assets_control_panel(){
		if( isset( $_GET['fdp-in-iframe'] ) ) return;
		$this->page_title = absint( $this->post_id ) > 0 ? esc_html( get_the_title( $this->post_id ) ) : '';
		require_once EOS_DP_PLUGIN_DIR.'/admin/eos-dp-helper.php';
		?>
		<div id="fdp-assets-main-wrp" style="left:-300px">
			<div id="fdp-assets-content">
				<h2><a class="fdp-page-view" href="<?php the_permalink( absint( $this->post_id ) ); ?>" target="_blank"><?php echo $this->page_title; ?></a></h2>
				<div class="fdp-save-wrp">
					<span class="fdp-assets-save fdp-button-active fdp-assets-button" data-action="eos_dp_pro_save_assets" data-nonce="fdp-save-assets" data-action-id="<?php echo uniqid(); ?>"><?php esc_html_e( 'Save','fdp-pro' ); ?></span>
					<div id="fdp-message-success" class="fdp-message fdp-success fdp-hidden"><?php esc_html_e( 'Options saved successfully.','eos-dp-pro' ); ?></div>
					<div id="fdp-message-error" class="fdp-message fdp-error fdp-hidden"><?php esc_html_e( 'Something went wrong.','eos-dp-pro' ); ?></div>
				</div>
				<div class="fdp-assets-control-panel">
					<?php
					if( 'off' === $this->status ){
						?>
						<div class="fdp-notice fdp-error" style="margin-top:32px">
							<?php esc_html_e( "You haven't the rights to manage the assets!","'eos-dp-pro" ); ?>
							<p style="margin-top:32px"><a class="fdp-button fdp-button-active" href="<?php echo wp_login_url( eos_dp_get_current_page_url() ); ?>" style="padding:6px 4px"><?php esc_html_e( 'Log in','eos-dp-pro' ); ?></a></p>
						</div>
					<?php }else{
					$labels = array( 'styles' => __( 'the stylesheets','eos-dp-pro' ),'scripts' => __( 'the JS scripts','eos-dp-pro' ) );
					?>
					<div style="margin-bottom:16px">
						<span id="fdp-assets-filter-global-view" class="fdp-needs-ajax fdp-assets-button">
							<a href="#"><?php esc_html_e( 'Global View','fdp-pro' ); ?></a>
						</span>
						<span id="fdp-assets-filter-close" class="fdp-assets-button">
							<a href="#"><?php esc_html_e( 'Close','fdp-pro' ); ?></a>
						</span>
						<span id="fdp-assets-reset-show" class="fdp-assets-button">
							<a href="#"><?php esc_html_e( 'Reset','fdp-pro' ); ?></a>
						</span>
						<div id="fdp-reset-msg-wrp" class="eos-hidden fdp-message fdp-warning">
							<p><?php printf( esc_html_x( 'Are you sure you want to reset all the settings related to %s?','Are you sure you want to reset all the settings related to %?','eos-dp-pro' ),$labels[sanitize_key( $this->status )] ); ?></p>
							<span id="fdp-assets-reset" class="fdp-assets-button fdp-needs-ajax">
								<a href="#"><?php esc_html_e( 'Yes reset all','fdp-pro' ); ?></a>
							</span>
						</div>
					</div>
					<div>
						<span>
							<span id="fdp-assets-filter-styles" class="fdp-assets-button<?php echo 'styles' === $this->status ? ' fdp-button-active' : ''; ?>">
								<?php esc_html_e( 'CSS','fdp-pro' ); ?>
							</span>
							<span id="fdp-assets-filter-scripts" class="fdp-assets-button<?php echo 'scripts' === $this->status ? ' fdp-button-active' : ''; ?>">
								<?php esc_html_e( 'JS','fdp-pro' ); ?>
							</span>
						</span>
					</div>
					<div style="margin-bottom:32px;margin-top:16px">
						<h1 id="fdp-page-title"><?php echo esc_html( $this->page_title ); ?></h1>
						<span id="fdp-collapse-sidebar" class="fdp-move-sidebar dashicons dashicons dashicons-arrow-left" title="<?php esc_attr_e( 'Collapse','eos-dp-pro' ); ?>"></span>
						<span id="fdp-expand-sidebar" class="fdp-move-sidebar dashicons dashicons-arrow-right" title="<?php esc_attr_e( 'Expand','eos-dp-pro' ); ?>"></span>
					</div>
					<div>
						<p style="margin-bottom:16px">
							<span id="fdp-assets-filter-all" class="fdp-assets-filter fdp-button-active fdp-assets-button" data-filter="all"><?php esc_html_e( 'All','fdp-pro' ); ?></span>
							<span id="fdp-assets-filter-core" class="fdp-assets-filter fdp-assets-button" data-filter="core">
							<span class="dashicons dashicons-wordpress-alt"></span><?php esc_html_e( 'Core','fdp-pro' ); ?></span>
							<span id="fdp-assets-filter-plugins" class="fdp-assets-filter fdp-assets-button" data-filter="plugin">
								<span class="dashicons dashicons-admin-plugins"></span><?php esc_html_e( 'Plugins','fdp-pro' ); ?>
							</span>
							<span id="fdp-assets-filter-theme" class="fdp-assets-filter fdp-assets-button" data-filter="theme">
								<span class="dashicons dashicons-admin-appearance"></span><?php esc_html_e( 'Theme','fdp-pro' ); ?>
							</span>
						</p>
						<p style="margin-bottom:16px">
							<select class="fdp-asset-plugins-sel" style="padding-top:5px;padding-bottom:2px">
								<option value="all"><?php esc_html_e( 'All plugins','eos-dp-pro' ); ?></option>
								<option value="active"><?php esc_html_e( 'Active plugins','eos-dp-pro' ); ?></option>
								<option value="disabled"><?php esc_html_e( 'Disabled plugins','eos-dp-pro' ); ?></option>
								<option value="void" disabled></option>
							<?php
							$active_plugins = eos_dp_active_plugins();
							foreach( $active_plugins as $plugin ){
								$plugin_slug = dirname( $plugin );
								$plugin_name = ucwords( str_replace( '-',' ',$plugin_slug ) );
								?>
								<option value="<?php echo esc_attr( $plugin_slug ); ?>"><?php echo esc_html( $plugin_name ); ?></option>
								<?php
							}
							?>
							</select>
						</p>
						<div class="right" style="margin-bottom:16px"><span id="fdp-prevew-assets" class="fdp-assets-button" data-action="eos_dp_pro_preview_assets" data-nonce="fdp-preview-assets" data-action-id="<?php echo uniqid(); ?>">
							<span><?php esc_html_e( 'Preview','eos-dp-pro' ); ?></span>
							<span class="dashicons dashicons-search" style="font-size:25px !important"></span>
						</div>
						<div id="fdp-assets-global-view-iframe-wrp" class="eos-hidden">
							<p class="right" style="margin-bottom:-10px;padding-top:10px"><span id="fdp-close-assets-iframe" class="hover dashicons dashicons-no-alt"></span></p>
							<iframe id="fdp-assets-global-view-iframe" src="?fdp-in-iframe=true" data-src="<?php echo add_query_arg( array( 'page' => 'eos_dp_pro_assets','asset_type' => $this->status,'fdp_in_iframe' => 'true' ),admin_url( 'admin.php' ) ); ?>"></iframe>
						</div>
					</div>
				<?php } ?>
				</div>
		<?php
		wp_nonce_field( 'fdp-reset-assets','fdp-reset-assets' );
		wp_nonce_field( 'fdp-save-assets','fdp-save-assets' );
		wp_nonce_field( 'fdp-preview-assets','fdp-preview-assets' );
		wp_nonce_field( 'fdp_pro_reduce_css','fdp_pro_reduce_css' );
	}

	function render_assets( $type,$position ){
		global $eos_dp_paths;
		$asset_icons = array(
			'core' => ' dashicons-wordpress-alt',
			'plugin' => ' dashicons-admin-plugins',
			'theme' => ' dashicons-admin-appearance'
		);
		$disabled_plugins = $eos_dp_paths && is_array( $eos_dp_paths ) && !empty( $eos_dp_paths ) ? array_map( 'dirname',$eos_dp_paths ) : array();
		$in_footer_array = $this->assets_in_footer;
		$page_title = $this->page_title;
		$from_type_labels = array( 'core' => __( 'Loaded by core','eos-dp-pro' ),'plugin' => __( 'Loaded by %s','eos-dp-pro' ),'theme' => __( 'Loaded by %s','eos-dp-pro' ) );
		$post_types = get_post_types( array( 'publicly_queryable' => true,'public' => true ),'names','or' );
		$exclude_post_types = array( 'attachment','eos_custom_css' );
		foreach( $post_types as $post_type ){
			if( in_array( $post_type,$exclude_post_types ) ){
				unset( $post_types[$post_type] );
			}
		}
		$post_types_labels = array();
		$validate_url_base = 'https://jigsaw.w3.org/css-validator/validator?uri=';
		foreach( $post_types as $post_type ){
			$labsObj = get_post_type_object( $post_type );
			if( isset( $labsObj->labels ) ){
				$labs = $labsObj->labels;
				$post_types_labels[$post_type] = isset( $labs->name ) ? $labs->name : $post_type;
			}
		}
		$reduced_css = false;
		$options_assets = $this->options_assets;
		$custom_urls = isset( $options_assets['custom_urls'] ) ? json_decode( sanitize_text_field( stripslashes( $options_assets['custom_urls'] ) ),true ) : false;
		$reduced_opts = isset( $options_assets['reduced_styles'] ) ? $options_assets['reduced_styles'] : false;
		$reduced_styles_ids = isset( $reduced_opts[$this->post_id] ) ? $reduced_opts[$this->post_id] : false;
		$moved_footer = isset( $options_assets['moved_footer'] ) ? $options_assets['moved_footer'] : false;
		$moved_head = isset( $options_assets['moved_head'] ) ? $options_assets['moved_head'] : false;
		$moved_footer_styles_ids = isset( $moved_footer[$this->post_id] ) ? $moved_footer[$this->post_id] : false;
		$moved_head_styles_ids = isset( $moved_head[$this->post_id] ) ? $moved_head[$this->post_id] : false;
		$current_post_type = get_post_type( absint( $this->post_id ) );
		$single_exceptions = $this->post_id  && absint( $this->post_id  ) > 0 ? json_decode( sanitize_text_field( stripslashes( get_post_meta( $this->post_id,'_fdp_'.sanitize_key( $this->status ).'_active',true ) ) ),true ) : array();
		$taxs = get_taxonomies( array(),'objects' );
		?>
		<div style="margin-bottom:16px">
		<?php
		$n = 1;
		foreach( $this->assets as $handle => $src ){
			if( $src && '' !== $src && 'fdp-assets' !== $handle ){
				$custom_url = isset( $custom_urls[$handle] ) ? $custom_urls[$handle] : '';
				$in_head_can = is_array( $this->assets_head ) && in_array( $handle,$this->assets_head );
				$enqueued[] = $handle;
				$handle_opts = isset( $options_assets[$handle] ) ? $options_assets[$handle] : array();
				$handle_singles_opts = isset( $handle_opts['single'] ) && is_string( $handle_opts['single'] ) ? explode( ',',rtrim( $handle_opts['single'],',' ) ) : array();
				$handle_post_types_opts = isset( $handle_opts['post_types'] ) ? explode( ',',rtrim( $handle_opts['post_types'],',' ) ) : array();
				$is_single_active = !is_array( $handle_singles_opts ) || !in_array( $this->post_id,$handle_singles_opts );
				if( $current_post_type && in_array( $current_post_type,$handle_post_types_opts ) ){
					$is_single_active = false;
				}
				if( is_array( $single_exceptions ) && in_array( $handle,$single_exceptions ) ){
					$is_single_active = true;
				}
				$handle_archives_opts = isset( $handle_opts['archives'] ) ? $handle_opts['archives'] : array();
				if( !is_array( $handle_archives_opts ) ){
					$handle_archives_opts = explode( ',',$handle_archives_opts );
				}
				$handle_terms_opts = isset( $handle_opts['terms'] ) ? $handle_opts['terms'] : array();
				if( !is_array( $handle_terms_opts ) ){
					$handle_terms_opts = explode( ',',$handle_terms_opts );
				}
				$from_type = 'core';
				$from = '';
				$in_footer = (
						( is_array( $in_footer_array ) && in_array( $handle,$in_footer_array ) )
						|| ( is_array( $moved_footer_styles_ids ) && in_array( $handle,$moved_footer_styles_ids ) )
					)
					&& ( !is_array( $moved_head_styles_ids ) || !in_array( $handle,$moved_head_styles_ids ) );
				if( false !== strpos( $src,EOS_DP_PRO_PLUGINS_URL ) ){
					$from_type = 'plugin';
					$from = dirname( str_replace( EOS_DP_PRO_PLUGINS_URL,'',$src ) );
				}
				elseif( false !== strpos( $src,get_parent_theme_file_uri() ) ){
					$from_type = 'theme';
					$from = dirname( str_replace( get_theme_root_uri(),'',$src ) );
				}
				elseif( false !== strpos( $src,get_stylesheet_directory_uri() ) ){
					$from_type = 'theme';
					$from = dirname( str_replace( get_theme_root_uri(),'',$src ) );
				}
				if( '' !== $from ){
					$fromA = explode( '/',ltrim( $from,'/' ) );
					$from = $fromA[0];
				}
				if( false === strpos( $src,get_site_url() ) && '//' !== substr( $src,0,2 ) ){
					$src = get_site_url().'/'.$src;
				}
				$siteUrlA = explode( '//',get_site_url() );
				$srcA = explode( '//',$src );
				$srcA = explode( '?',$srcA[1] );
				$path = str_replace( $siteUrlA[1],ABSPATH,$srcA[0] );
				$full_path = str_replace( '//','/',str_replace( get_site_url().'/',ABSPATH,$src ) );
				$file_size = file_exists( $full_path ) ? esc_html( sprintf( '%s kB',round( filesize( $full_path )/1024,1 ) ) ) : 'unknown';
				$reduced_css = is_array( $reduced_styles_ids ) && in_array( $handle,$reduced_styles_ids );
				$data_src = '';
				if( $reduced_css ){
					$upload_dir = wp_upload_dir();
					$data_src = ' data-src="'.esc_attr( $upload_dir['baseurl'].'/FDP/css/fdp-'.$this->post_id.'-'.substr( md5( $this->post_id.$handle ),0,8 ) ).'.css"';
				}
				$file_path = false === strpos( get_site_url(),$srcA[0] ) ? str_replace( $siteUrlA[1],'',esc_attr( $srcA[0] ) ) : str_replace( '//','',str_replace( get_site_url(),'',$src ) );
				$code_browser_url = add_query_arg( array( 'id' => esc_attr( $handle ),'page' => 'eos_dp_code_browser','from_type' => $from_type,'from' => esc_attr( $from ),'path' => $file_path ),admin_url() );
				$plugin_active_class = '';
				$asset_icon = $asset_icons[$from_type];
				if( 'plugin' === $from_type ){
					if( in_array( $from,$disabled_plugins ) ){
						$plugin_active_class =  ' fdp-plugin-disabled';
					}
					else{
						$plugin_active_class =  ' fdp-plugin-active';
						$asset_icon = ' dashicons-plugins-checked';
					}
				}
			?>
			<span class="fdp-asset<?php echo $is_single_active ? ' fdp-active' : '';echo $plugin_active_class; ?> fdp-asset-<?php echo $type;echo $reduced_css ? ' fdp-reduced-style' : '';echo $in_footer ? ' fdp-in-footer' : ' fdp-in-head';echo $in_head_can ? ' fdp-in-head-can' : ' fdp-in-head-can-not'; ?>"<?php echo $data_src; ?> data-original-src="<?php echo esc_url( $src ); ?>" data-id="<?php echo esc_attr( $handle ); ?>" data-from-type="<?php echo $from_type; ?>" data-from="<?php echo esc_attr( $from ); ?>" data-position="<?php echo $in_footer ? 'fdp-in-footer' : 'fdp-in-head'; ?>" data-asset="<?php echo esc_attr( $this->status ); ?>">
				<span class="fdp-asset-n"><?php echo $n; ?>) </span>
				<span class="fdp-asset-icon dashicons<?php echo $asset_icon; ?>"></span>
				<?php echo esc_html( $handle ); ?>
				<div class="fdp-asset-actions">
					<?php if( 'styles' === $this->status ){ ?>
					<span title="<?php esc_html_e( 'Disable this stylesheet on this page','eos-dp-pro' ); ?>" class="fdp-needs-ajax fdp-asset-remove-toggle fdp-active-remove dashicons dashicons-yes-alt"></span>
					<span title="<?php esc_html_e( 'Activate this stylesheet on this page','eos-dp-pro' ); ?>"class="fdp-needs-ajax fdp-asset-remove-toggle fdp-active-activate dashicons dashicons-no-alt"></span>
					<?php if( defined( 'FDP_BETA_VERSION' ) && true === FDP_BETA_VERSION ){ ?>
					<a title="<?php esc_html_e( 'Validate CSS','eos-dp-pro' ); ?>"class="dashicons dashicons-code-standards" href="<?php echo $validate_url_base.urlencode( esc_url( $src ) ); ?>" rel="noopener" target="_blank"></a>
					<span title="<?php esc_html_e( 'Reduce CSS','eos-dp-pro' ); ?>" class="fdp-needs-ajax fdp-assets-reduce-css-button dashicons dashicons-fullscreen-exit-alt<?php echo $reduced_css ? ' fdp-hidden fdp-open' : ''; ?>"<?php echo $data_src; ?>></span>
					<span title="<?php esc_html_e( 'Restore original CSS','eos-dp-pro' ); ?>" class="fdp-assets-restore-css-button dashicons dashicons-editor-expand<?php echo !$reduced_css ? ' fdp-hidden' : ''; ?>"></span>
					<?php } ?>
					<?php } ?>
					<a title="<?php esc_html_e( 'Show file','eos-dp-pro' ); ?>"class="fdp-asset-file-link dashicons dashicons-info" href="<?php echo esc_url( $code_browser_url ); ?>" target="_blank"></a>
					<?php if( 'scripts' === $this->status ){ ?>
						<span class="fdp-assets-head-button dashicons dashicons-arrow-up-alt" title="<?php esc_html_e( 'Script in head','eos-dp-pro' ); ?>"></span>
						<span class="fdp-assets-footer-button dashicons dashicons-arrow-down-alt" title="<?php esc_html_e( 'Script in footer','eos-dp-pro' ); ?>"></span>
						<span class="fdp-assets-reset-button dashicons dashicons-undo" title="<?php esc_html_e( 'Reset position settings','eos-dp-pro' ); ?>" data-position-original="<?php echo in_array( $handle,$in_footer_array ) ? 'fdp-in-footer' : 'fdp-in-head'; ?>"></span>
					<?php } ?>
					<span class="fdp-assets-edit-button dashicons dashicons-edit" title="<?php esc_html_e( 'Edit options','eos-dp-pro' ); ?>"></span>
				</div>
				<div class="fdp-assets-edit fdp-hidden" data-asset="<?php echo esc_attr( $handle ); ?>">
					<div style="position:relative">
						<span class="fdp-close-popup fdp-button" data-button="fdp-assets-edit-button" style="position:sticky;top:0"><?php esc_html_e( 'Close','eos-dp-pro' ); ?></span>
						<div><h2 style="margin-top:12px;margin-bottom:16px"><?php echo esc_html( $handle ); ?></strong></h2></div>
						<div>
							<span class="fdp-legend-txt"><span class="fdp-button-legend fdp-button-active"></span><?php esc_html_e( 'Load','eos-dp-pro'); ?> </span>
							<span class="fdp-legend-txt"><span class="fdp-button-legend fdp-button"></span> <?php esc_html_e( 'Unload','eos-dp-pro'); ?></span>
						</div>
						<?php if( absint( $this->post_id ) > 0 ){ ?>
						<div class="fdp-asset-edit-section fdp-asset-singles" style="min-height:40px">
							<p><strong><?php esc_html_e( 'This page','eos-dp-pro' ); ?></strong></p>
							<span class="fdp-button<?php echo $is_single_active ? ' fdp-button-active' : ''; ?>" data-where-type="single" data-where="<?php echo esc_attr( $this->post_id ); ?>" data-post_type="<?php echo esc_attr( $current_post_type ); ?>"><?php echo esc_html( $page_title ); ?></span>
						</div>
						<?php } ?>
						<div class="fdp-asset-edit-section fdp-asset-custom-url" style="min-height:40px">
							<p><strong><?php esc_html_e( 'Unload by custom URL','eos-dp-pro' ); ?></strong></p>
							<p><?php esc_html_e( 'To unload an asset everywhere write only the star symbol *','eos-dp-pro' ); ?></p>
							<textarea class="fdp-asset-custom-url-input" style="color:#000"><?php echo esc_html( $custom_url ); ?></textarea>
							<p><?php esc_html_e( '(Separate them by comma)','eos-dp-pro' ); ?></p>
						</div>
						<div class="fdp-asset-edit-section fdp-asset-post-types">
							<p><strong><?php esc_html_e( 'Post Types','eos-dp-pro' ); ?></strong></p>
							<?php foreach( $post_types_labels as $post_type => $label ){ ?>
								<span class="fdp-button<?php echo !is_array( $handle_post_types_opts ) || !in_array( $post_type,$handle_post_types_opts ) ? ' fdp-button-active' : ''; ?>" data-where-type="post_types" data-where="<?php echo esc_attr( $post_type ); ?>"><?php echo esc_html( $label ); ?></span>
							<?php } ?>
						</div>
						<div class="fdp-asset-edit-section fdp-asset-archives">
							<p><strong><?php esc_html_e( 'Archives','eos-dp-pro' ); ?></strong></p>
							<?php
							foreach( $post_types_labels as $post_type => $label ){
							?>
							<span class="fdp-button<?php echo !is_array( $handle_archives_opts ) || !in_array( $post_type,$handle_archives_opts ) ? ' fdp-button-active' : ''; ?>" data-where-type="archives" data-where="<?php echo esc_attr( $post_type ); ?>"><?php echo esc_html( sprintf( __( '%s archive','eos-dp-proo' ),$label ) ); ?></span>
						<?php } ?>
						</div>


						<div class="fdp-asset-edit-section fdp-asset-terms">
						  <p><strong><?php esc_html_e( 'Terms','eos-dp-pro' ); ?></strong></p>
						<?php
						if( $taxs ){
						  foreach( $taxs as $tax ){
						    if( '1' == $tax->public && isset( $tax->object_type ) ){
						      $show = false;
						      $labels_names = array();
						      foreach( $tax->object_type as $term_post_type ){
						        if( in_array( $term_post_type,$post_types ) ){
											?>
							        <span id="fdp-term-<?php echo esc_attr( $tax->name ); ?>" class="fdp-button<?php echo !is_array( $handle_terms_opts ) || !in_array( $tax->name,$handle_terms_opts ) ? ' fdp-button-active' : ''; ?>" data-where-type="terms" data-where="<?php echo esc_attr( $tax->name ); ?>">
												<?php echo esc_html( sprintf( __( '%s (%s)','eos-dp-proo' ),$tax->label,$term_post_type ) ); ?>
											</span>
							        <?php
						        }
						      }
						    }
						  }
						}
						?>
						</div>
					</div>
				</div>
				<?php if( 'unknown' !== $file_size ){ ?>
				<span class="fdp-asset-filesize"><?php echo esc_html( $file_size ); ?></span>
				<?php } ?>
			</span>
			<?php
			++$n;
			}
		}
		?>
			</div>
		</div>
		<script>
		var fdp_body = document.getElementsByTagName('body')[0];
		fdp_body.className = fdp_body.className.replace( ' fdp-assets-loaded','') + ' fdp-assets-loaded';
		document.getElementById('fdp-manage-links').innerHTML = '.fdp-assets-loaded a{pointer-events:initial}';
		</script>
		<?php
	}

	function get_assets( $type ){
		if( !isset( $GLOBALS['wp_'.$type] ) ) return array();
		$assets = array();
		$exclude = array(
			'admin-bar',
			'fdp-dashicons',
			'fdp-assets-js'
		);
		foreach( $GLOBALS['wp_'.$type]->queue as $asset ){
			if( !in_array( $asset,$exclude ) ){
			 $assets[$asset] =  $GLOBALS['wp_'.$type]->registered[$asset]->src;
		 	}
		}
		return $assets;
	}
}
$fdp_assets = new FDP_PRO_Show_Assets();
