<?php
defined( 'ABSPATH' ) || exit;
/*
* This class is a fork of the class ABC_Show_Hooks of the plugin Show Hooks https://wordpress.org/plugins/show-hooks/
* Author of the plugin Show Hooks: rafiq91 https://profiles.wordpress.org/rafiq91/
*/
class FDP_PRO_Show_Hooks {
	private $status;
	private $all_hooks = array();
	private $recent_hooks = array();
	private $ignore_hooks = array();
	private $doing = 'false';
	private $post_id = false;

	public function __construct() {
		// Will be display in the popup
		$skip_hooks = array( 'attribute_escape' );
		if( isset( $_REQUEST['skip_hooks'] ) ){
			$skip_hooks = array_merge( $skip_hooks,explode( ';',esc_attr( $_REQUEST['skip_hooks'] ) ) );
		}
		$this->ignore_hooks = apply_filters( 'fdp_skip_hooks', $skip_hooks );
		$this->set_active_status();
		$this->attach_hooks();
		// Init the plugin.
		add_action( 'init', array( $this,'plugin_init' ) );
		add_filter( 'body_class', array( $this,'body_class' ) );
	}

	//It sets the active status of the hooks displaying.
	public function set_active_status() {
		if ( ! isset( $this->status ) ) {
			if ( isset( $_REQUEST['fdp-hooks'] ) ) {
				$this->status = sanitize_text_field( $_REQUEST['fdp-hooks'] );
			}
			else{
				$this->status = 'off';
			}
		}
	}

	//It attaches the filter that render all the hook labels.
	public function attach_hooks(){
		if ( $this->status == 'actions' || $this->status == 'filters' ) {
			add_filter( 'all', array( $this, 'hook_all_hooks' ), 100 );
			add_action( 'shutdown', array( $this, 'filter_hooks_panel' ) );
		}
	}

	//Initialization
	function plugin_init(){
		if ( !current_user_can( 'activate_plugins' ) && !current_user_can( 'fdp_plugins_viewer' ) ) {
			$this->status = 'off';
			return;
		}
		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_script' ) );
		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_script' ) );
		if ( $this->status == 'actions' || $this->status == 'filters' ) {
			add_action( 'wp_head', array( $this, 'render_head_hooks'), 100 );
			add_action( 'admin_head', array( $this, 'render_head_hooks'), 100 );
		}
	}

	public function body_class( $classes ){
		$classes[] = 'fdp-hooks-page';
		return $classes;
	}
	//Enqueue Scripts
	public function enqueue_script($screen){
		global $post;
		$this->post_id = is_object( $post ) ? $post->ID : 0;
		wp_enqueue_style( 'dashicons' );
		wp_enqueue_style( 'fdp-hooks', EOS_DP_PRO_PLUGIN_URL. '/assets/css/fdp-show-hooks.css','',EOS_DP_PRO_VERSION,'all' );
		wp_enqueue_script( 'fdp-main-js', EOS_DP_PRO_PLUGIN_URL . '/assets/js/fdp-show-hooks.js', array( 'jquery' ),EOS_DP_PRO_VERSION,true );
		wp_localize_script('fdp-main-js', 'fdp_hooks', array(
			'post_id' => $this->post_id,
			'ajaxurl' => admin_url('admin-ajax.php')
		));
	}
	function render_head_hooks() {
		$this->render_hooks();
		$this->doing = 'write';
	}

	//Render all hooks
	function render_hooks(){
		$this->hooks_control_panel();
		foreach ( $this->all_hooks as $nested_value ) {
			if ( 'action' == $nested_value['type'] ) {
				$this->render_action( $nested_value );
			}
		}
	}

	function hooks_control_panel(){
		?>
		<div id="fdp-hooks-control-panel">
			<h1 id="fdp-page-title" style="margin:0;display:inline-block"><a href="<?php echo get_permalink(); ?>" target="_blank"><?php echo isset( $_GET['fdp_post_id'] ) ? esc_html( get_the_title( $_GET['fdp_post_id'] ) ) : ''; ?></a></h1>
			<div style="float:<?php echo is_rtl() ? 'left' : 'right'; ?>">
				<span id="fdp-show-actions" class="fdp-button-active fdp-hooks-button"><?php esc_html_e( 'Actions','fdp-pro' ); ?></span>
				<span id="fdp-show-filters" class="fdp-hooks-button"><?php esc_html_e( 'Filters','fdp-pro' ); ?></span>
				<span id="fdp-show-empty" class="fdp-hooks-button"><?php esc_html_e( 'Not assigned','fdp-pro' ); ?></span>
				<?php if( defined( 'FDP_PRO_DEACTIVATE_HOOKS' ) && FDP_PRO_DEACTIVATE_HOOKS ){ ?>
				<a id="fdp-hooks-global-view" class="fdp-hooks-button" href="<?php echo admin_url( 'admin.php?page=eos_dp_pro_removed_hooks' ); ?>" target="_blank"><?php esc_html_e( 'Global site view','fdp-pro' ); ?></a>
				<?php } ?>
			</div>
		</div>
		<?php
		wp_nonce_field( 'fdp-save-hooks','fdp-save-hooks' );
	}
	//Hook all hooks
	public function hook_all_hooks( $hook ) {
		global $wp_actions, $wp_filter;
		if ( ! in_array( $hook, $this->recent_hooks ) ) {
			if ( isset( $wp_actions[$hook] ) ) {
				// Action
				$this->all_hooks[] = array(
					'ID'       => $hook,
					'callback' => false,
					'type'     => 'action',
				);
			}
			else {
				// Filter
				$this->all_hooks[] = array(
					'ID'       => $hook,
					'callback' => false,
					'type'     => 'filter',
				);
			}
		}
		if ( isset( $wp_actions[$hook] ) && !in_array( $hook, $this->recent_hooks ) && !in_array( $hook, $this->ignore_hooks ) ) {
			if ( 'write' == $this->doing ) {
				$this->render_action( end( $this->all_hooks ) );
			}
		}
		$this->recent_hooks[] = $hook;
		if ( count( $this->recent_hooks ) > 100 ) {
			array_shift( $this->recent_hooks );
		}
	}

	//Render
	function render_action( $args = array() ){
		if( doing_filter( 'body_class' ) ) return;
		global $wp_filter;
		// Get all the nested hooks
		$nested_hooks = ( isset( $wp_filter[ $args['ID'] ] ) ) ? $wp_filter[ $args['ID'] ] : false ;
		// Count the number of functions on this hook
		$nested_hooks_count = 0;
		if( $nested_hooks ){
			foreach( $nested_hooks as $key => $value ){
				$nested_hooks_count += count( $value );
			}
		}
		$removed_hooks = false;
		$fdp_pro_opts = eos_dp_get_option( 'eos_dp_pro_main' );
		if( $fdp_pro_opts && isset( $fdp_pro_opts['fdp_hooks'] ) ){
			$hooks = $fdp_pro_opts['fdp_hooks'];
			if( !empty( $hooks ) && isset( $hooks[absint( $this->post_id )] ) ){
				$removed_hooks = json_decode( sanitize_text_field( stripslashes( $hooks[absint( $this->post_id )] ) ),true );
			}
		}
		$icons = array(
			'core' => 'wordpress-alt',
			'plugin' => 'admin-plugins',
			'mu-plugin' => 'admin-plugins',
			'theme' => 'admin-appearance',
			'child-theme' => 'admin-appearance',
			'unkonwn' => 'editor-help'
		);
		?>
		<span data-id="<?php echo esc_attr( $args['ID'] ); ?>" style="display:none;" class="fdp-hook fdp-hook-<?php echo esc_attr($args['type']);echo isset( $_GET['hook_name'] ) && $args['ID'] === $_GET['hook_name'] ? ' fdp-current-hook' : '';echo ( $nested_hooks ) ? ' fdp-hook-has-hooks' : ' fdp-hook-empty' ; ?>" >
			<?php
			echo esc_attr( $args['ID'] );
			// Write the count number if any function are hooked.
			if( $nested_hooks_count ){ ?>
				<span class="fdp-hook-count" data-count="<?php echo esc_attr( $nested_hooks_count ); ?>"> (<?php echo esc_attr( $nested_hooks_count ); ?>)</span>
				<?php
			}
			// Write out list of all the function hooked to an action.
			if( isset( $wp_filter[$args['ID']] ) ):
				$nested_hooks = $wp_filter[$args['ID']];
				if( $nested_hooks ):
				?>
				<div class="fdp-overlap" style="display:none;position:fixed;top:0;bottom:0;left:0;right:0;background:#000;background:rgba(0,0,0,0.7)">
					<div class="fdp-hook-dropdown">
						<div class="fdp-hooks-ul-actions" style="top:0;background:#fff">
							<strong id="fdp-hook-title"><a href="https://developer.wordpress.org/reference/hooks/<?php echo esc_attr( $args['ID'] ); ?>" target="_blank" rel="noopener"><?php echo esc_attr( $args['ID'] ); ?></a></strong>
							<span class="fdp-filter-hook-row-all fdp-hooks-button dashicons" style="width:auto" title="<?php esc_attr_e( 'All','eos-dp-pro' ); ?>"><?php esc_html_e( 'All','fdp-pro' ); ?></span>
							<span class="fdp-filter-hook-row dashicons dashicons-wordpress-alt fdp-hooks-button" data-row="core" title="<?php esc_attr_e( 'Core','eos-dp-pro' ); ?>"></span>
							<span class="fdp-filter-hook-row dashicons dashicons-admin-plugins fdp-hooks-button" data-row="plugin" title="<?php esc_attr_e( 'Plugin','eos-dp-pro' ); ?>"></span>
							<span class="fdp-filter-hook-row dashicons dashicons-admin-appearance fdp-hooks-button" data-row="theme" title="<?php esc_attr_e( 'Theme','eos-dp-pro' ); ?>"></span>
							<?php if( defined( 'FDP_PRO_DEACTIVATE_HOOKS' ) && FDP_PRO_DEACTIVATE_HOOKS ){ ?>
							<span class="fdp-activate-all-ul fdp-hooks-button"><?php esc_html_e( 'Activate all','fdp-pro' ); ?></span>
							<span class="fdp-disable-all-ul fdp-hooks-button"><?php esc_html_e( 'Disable all','fdp-pro' ); ?></span>
							<span class="fdp-save-ul fdp-hooks-button"><?php esc_html_e( 'Save','fdp-pro' ); ?></span>
							<?php } ?>
							<span class="fdp-close-ul fdp-hooks-button dashicons" style="width:auto"><?php esc_html_e( 'Close','fdp-pro' ); ?></span>
						</div>
						<table class="fdp-hook-table" style="cursor:default">
							<thead>
								<tr>
									<th><?php esc_html_e( 'Function','eos-dp-pro' ); ?></th>
									<th><?php esc_html_e( 'Priority','eos-dp-pro' ); ?></th>
									<th><?php esc_html_e( 'Source','eos-dp-pro' ); ?></th>
									<th><span class="dashicons dashicons-text-page"></span></th>
								</tr>
							</thead>
							<tbody>
							<?php
							foreach( $nested_hooks as $priority => $nested_value ){
								// Show the priority number if the following hooked functions
								foreach ( $nested_value as $nested_inner_key => $nested_inner_value ){
										// Show all teh functions hooked to this priority of this hook
										$function_name_class = false;
										if ( $nested_inner_value['function'] && is_array( $nested_inner_value['function'] ) && count( $nested_inner_value['function'] ) > 1 ){
											$function_name = '';
											// Hooked function ( of type object->method() )
											$classname = false;
											if ( is_object( $nested_inner_value['function'][0] ) || is_string( $nested_inner_value['function'][0] ) ) {
												if ( is_object( $nested_inner_value['function'][0] ) ) {
													$classname = get_class($nested_inner_value['function'][0] );
												}
												if ( is_string( $nested_inner_value['function'][0] ) ) {
													$classname = $nested_inner_value['function'][0];
												}
												if ( $classname ) {
													$function_name_class = $classname;
													$function_name .= esc_attr( $classname ).'&ndash;&gt;';
												}
											}
											$function_name .= esc_attr( $nested_inner_value['function'][1] );
										}
										else{
											$function_name = esc_attr( $nested_inner_key );
										}
										$is_removed = false;
										if( $removed_hooks && isset( $removed_hooks[$args['ID']] ) ){
											foreach( $removed_hooks[$args['ID']] as $arr_hook ){
												if( str_replace( '–&gt;','',esc_attr( $arr_hook[0] ) ) === str_replace( '&ndash;&gt;','',$function_name ) && absint( $arr_hook[2] ) === $this->post_id ){
													$is_removed = true;
													break;
												}
											}
										}
										$function_who = $function_name_class ? $this->function_who( $function_name_class ) : $this->function_who( $function_name );
										$file_browser_url = add_query_arg( array( 'id' => $function_name,'page' => 'eos_dp_code_browser','path' => $function_who && is_array( $function_who ) ? $function_who['path'] : '' ),admin_url() );
										if( isset( $function_who['type'] ) && '' !== $function_who['type'] ){
											$hook_class = $is_removed ? 'fdp-hook-removed' : 'fdp-hook-active';
											$hook_class = defined( 'FDP_PRO_DEACTIVATE_HOOKS' ) && FDP_PRO_DEACTIVATE_HOOKS ? $hook_class : '';
										?>
										<tr class="fdp-hook-row fdp-hook-row-<?php echo esc_attr( $function_who['type'] ); ?>">
											<td class="<?php echo $hook_class; ?>">
													<span class="<?php echo defined( 'FDP_PRO_DEACTIVATE_HOOKS' ) && FDP_PRO_DEACTIVATE_HOOKS ? 'fdp-function-string' : ''; ?>" data-hook_name="<?php echo esc_attr( $args['ID'] ); ?>" data-priority="<?php echo esc_attr( $priority ); ?>">
														<?php echo  esc_html( $function_name ); ?>
													</span>
											</td>
											<td class="fdp-priority"><?php echo esc_attr( $priority ); ?></td>
											<td class="fdp-from"><span class="dashicons dashicons-<?php echo isset( $icons[$function_who['type']] ) ? $icons[$function_who['type']] : ''; ?>"><?php echo 'mu-plugin' === $function_who['type'] ? ' MU' : ''; ?></span><?php echo ucwords( str_replace( '-',' ',$function_who['who'] ) ); ?></td>
											<td class="fdp-path"><p><span class="dashicons dashicons-media-document"></span></span><a href="<?php echo esc_url( $file_browser_url ); ?>" target="_blank" class="fdp-path-value"><?php echo esc_html( $function_who['path'] ); ?><a></p></td>
										</tr>
										<?php
									}
								}
							}
							?>
							</tbody>
						</table>
					</div>
				</div>
					<?php
				endif;
			endif;
			?>
		</span>
		<?php

	}

	//Hooks Panel
	function filter_hooks_panel() {
		global $wp_filter, $wp_actions;
		?>
		<div class="fdp-nested-hooks-block <?php echo ( 'filters' == $this->status ) ? esc_html('fdp-active') : '' ; ?> ">
			<div style="padding:10px;text-align:<?php echo is_rtl() ? 'left' : 'right'; ?>"><span id="fdp-close-hooks-popup" class="fdp-hooks-button"><?php esc_html_e( 'Close','fdp-pro' ); ?></span></div>
			<?php
			foreach ( $this->all_hooks as $va_nested_value ) {
				if ( 'filter' == $va_nested_value['type'] ) {
					$this->render_action( $va_nested_value );
				}
			}
			?>
		</div>
		<?php
	}
	//Return the file where a function or class is defined
	function function_who( $function_name ){
		if( function_exists( $function_name ) ){
			$reflector = new \ReflectionFunction( $function_name );
		}
		elseif( class_exists( $function_name ) ){
			$reflector = new \ReflectionClass( $function_name );
		}
		else{
			return false;
		}
		$output = array();
		$path = $reflector->getFileName();
		$plugindir = defined( PLUGINDIR ) ? PLUGINDIR.'/' : 'wp-content/plugins/';
		$muplugindir = str_replace( ABSPATH,'',EOS_DP_MU_PLUGIN_DIR ).'/';
		$themedir = str_replace( ABSPATH,'',get_theme_root() );
		if( false !== strpos( $path,$plugindir ) ){
			$path = str_replace( ABSPATH,'',$path );
			$output['type'] = 'plugin';
			$output['path'] = $path;
			$path = str_replace( $plugindir,'',$path );
			$whoA = explode( '/',$path );
			$output['who'] = $whoA[0];
		}
		elseif( false !== strpos( $path,$muplugindir ) ){
			$path = str_replace( ABSPATH,'',$path );
			$output['type'] = 'mu-plugin';
			$output['path'] = $path;
			$path = str_replace( $muplugindir,'',$path );
			$whoA = explode( '/',$path );
			$output['who'] = 'mu-plugin';
		}
		elseif( false !== strpos( $path,$themedir ) ){
			$childdir = get_stylesheet_directory();
			$parentdir = get_template_directory();
			if( false !== strpos( $path,$parentdir ) ){
				$output['type'] = 'theme';
			}
			if( false !== strpos( $path,$childdir ) ){
				$output['type'] = 'child-theme';
			}
			$output['who'] = dirname( str_replace( get_theme_root().'/','',$path ) );
			if( false !== strpos( $output['who'],'/' ) ) $output['who'] = dirname( $output['who'] );
			$output['path'] = str_replace( ABSPATH,'',$path );
		}
		else{
			$path = str_replace( ABSPATH,'',$path );
			if( 0 === strpos( $path,'wp-' ) ){
				$output['type'] = 'core';
				$output['who'] = 'core';
			}
			else{
				$output['type'] = 'unkown';
				$output['who'] = 'unkown';
			}
			$output['path'] = $path;
		}
		return array_map( 'esc_attr',$output );
	}
}
$fdp_hooks = new FDP_PRO_Show_Hooks();
