<?php

// Code to test a site via GTmetrix using the Web Testing
// Framework (WTF) API.
// Load the web test framework class.

if ( ! defined( 'ABSPATH' ) ) exit;
require_once( EOS_DP_PRO_PLUGIN_DIR.'/library/gtmetrix/Services_WTF_Test.php' );
function eos_dp_pro_call_gtmetrix( $post_id,$post_title,$url_to_test,$user_name,$api_key ){
	$opts = get_site_option( 'fdp_pro_admin' );
	$test = new Services_WTF_Test( $user_name,$api_key );
	$resultsA = array();
	$location = isset( $opts['gtmetrix_location_id'] ) ? absint( $opts['gtmetrix_location_id'] ) : 0;
	$testid = $test->test( array(
		'url' => $url_to_test,
		'location' => $location
	) );
	if( $testid && false !== strpos( $url_to_test,'&' ) ){
		$url_args = explode( '&',$url_to_test );
		unset( $url_args[0] );
		if( in_array( 'tool=gtmetrix',$url_args )  ){
			unset( $url_args[array_search( 'tool=gtmetrix',$url_args )] );
			$url_args = array_values( $url_args );
			if( false !== strpos( $url_args[0],'fdp_post_id' ) ){
				$plugins= array_filter( array_unique( explode( ';pn:',esc_attr( get_transient( 'fdp_test_gtmetrix_'.sanitize_key( str_replace( 'fdp_post_id=','',$url_args[0] ) ) ) ) ) ) );
				if( !empty( $plugins ) ){
						$plugins = array_map( 'eos_dp_name_by_path',$plugins );
						$resultsA['disabled_plugins'] = sanitize_text_field( implode( ',',$plugins ) );
				}
				$resultsA['post_id'] = $post_id;
				$resultsA['post_title'] = esc_html( $post_title );
				$resultsA['page_url'] = $url_to_test;
				$resultsA['location'] = $location;
				$resultsA['theme'] = sanitize_text_field( str_replace( '&theme=','',$theme ) );
			}
			else{
				die( "Test failed: something went wrong."  . "\n" );
			}
		}
		else{
			die( "Test failed: something went wrong."  . "\n" );
		}
	}
	else {
		die( "Test failed: " . $test->error() . "\n" );
	}
	$test->get_results();
	if( $test->error() ){
		die($test->error());
	}
	$results = $test->results();
	if( isset( $results['report_url'] ) ){
		$results['screenshot'] = 'https://gtmetrix.com/reports/'.sanitize_text_field( $_SERVER['SERVER_NAME'] ).'/'.sanitize_text_field( basename( $results['report_url'] ) ).'/screenshot.jpg';
	}
	$results['test_id'] = $testid;
	$results['tool'] = 'gtmetrix';
	echo wp_json_encode( $results );
	foreach( $results as $result => $data) {
		$resultsA[sanitize_key( $result )] = sanitize_text_field( $data );
	}
	$resources = $test->resources();
	foreach( $resources as $resource => $url ){
		$resultsA[sanitize_key( $resource )] = esc_url( $url );
	}
	$test->load( $testid );
	$result = $test->delete();
	if( !$result) {
		die( "error deleting test: " . $test->error());
	}
	$writeAccess = false;
	$access_type = get_filesystem_method();
	if( $access_type === 'direct' ){
		$creds = request_filesystem_credentials( EOS_WP_ADMIN, '', false, false, array() );
		if ( ! WP_Filesystem( $creds ) ) {
			return false;
		}
		global $wp_filesystem;
		$writeAccess = true;
	}
	$upload_dir = wp_upload_dir();
	if( true === $writeAccess ){
		eos_dp_pro_write_json_from_array( $resultsA,true );
		die();
		exit;
	}
	die();
	exit;
}
